// src/collapsed-frame-ui.tsx
import React, { useState } from "react";
var defaultTheme = {
  buttonBg: "#fff",
  buttonBorderColor: "#ccc",
  buttonHoverBg: "#efefef",
  buttonColor: "#444",
  buttonRadius: "4",
  bg: "#efefef"
};
var getThemeWithDefaults = (theme) => {
  return {
    ...defaultTheme,
    ...theme
  };
};
function CollapsedFrameUI({
  frameState,
  theme,
  FrameImage,
  allowPartialFrame
}) {
  const [isImageLoading, setIsImageLoading] = useState(true);
  const currentFrame = frameState.currentFrameStackItem;
  const isLoading = currentFrame?.status === "pending" || isImageLoading;
  const resolvedTheme = getThemeWithDefaults(theme ?? {});
  if (!frameState.homeframeUrl) {
    return null;
  }
  if (!currentFrame) {
    return null;
  }
  if (currentFrame.status === "done" && currentFrame.frameResult.status === "failure" && !(allowPartialFrame && // Need at least image and buttons to render a partial frame
  currentFrame.frameResult.frame.image && currentFrame.frameResult.frame.buttons)) {
    return null;
  }
  let frame;
  if (currentFrame.status === "done") {
    frame = currentFrame.frameResult.frame;
  } else if (currentFrame.status === "message" || currentFrame.status === "doneRedirect") {
    frame = currentFrame.request.sourceFrame;
  } else if (currentFrame.status === "requestError") {
    frame = "sourceFrame" in currentFrame.request ? currentFrame.request.sourceFrame : void 0;
  }
  const ImageEl = FrameImage ? FrameImage : "img";
  return /* @__PURE__ */ React.createElement(
    "div",
    {
      style: { backgroundColor: resolvedTheme.bg },
      className: "flex w-full gap-2 rounded p-2 items-center"
    },
    /* @__PURE__ */ React.createElement(
      "div",
      {
        className: "relative max-h-[60px] shrink-0",
        style: { aspectRatio: "1/1" }
      },
      " ",
      !!frame && !!frame.image && /* @__PURE__ */ React.createElement(
        ImageEl,
        {
          src: frame.image,
          key: frame.image,
          alt: "Frame image",
          width: "100%",
          style: {
            filter: isLoading ? "blur(4px)" : void 0,
            borderRadius: `${resolvedTheme.buttonRadius}px`,
            border: `1px solid ${resolvedTheme.buttonBorderColor}`,
            objectFit: "cover",
            width: "100%",
            aspectRatio: "1/1"
          },
          onLoadStart: () => {
            setIsImageLoading(true);
          },
          onLoad: () => {
            setIsImageLoading(false);
          },
          onError: () => {
            setIsImageLoading(false);
          }
        }
      )
    ),
    /* @__PURE__ */ React.createElement("div", { className: "flex flex-col flex-1 justify-center min-w-[0]" }, /* @__PURE__ */ React.createElement("span", { className: "font-semibold block truncate" }, frame?.title), /* @__PURE__ */ React.createElement("span", { className: "text-gray-500 text-xs block truncate" }, new URL(currentFrame.url).hostname)),
    !!frame && !!frame.buttons ? /* @__PURE__ */ React.createElement("div", { className: "flex items-center shrink-0" }, /* @__PURE__ */ React.createElement(
      "button",
      {
        type: "button",
        disabled: isLoading,
        className: `p-2 ${isLoading ? "bg-gray-100" : ""} border text-sm text-gray-800 rounded`,
        style: {
          flex: "1 1 0px",
          // fixme: hover style
          backgroundColor: resolvedTheme.buttonBg,
          borderColor: resolvedTheme.buttonBorderColor,
          color: resolvedTheme.buttonColor,
          cursor: isLoading ? void 0 : "pointer"
        }
      },
      frame.buttons.length === 1 && frame.buttons[0].label.length < 12 ? frame.buttons[0].label : "View"
    )) : null
  );
}

export {
  CollapsedFrameUI
};
