import {
  WebStorage
} from "./chunk-YWADTQF4.js";

// src/identity/xmtp/use-xmtp-identity.tsx
import { useConnectModal } from "@rainbow-me/rainbowkit";
import { FramesClient } from "@xmtp/frames-client";
import { Client } from "@xmtp/xmtp-js";
import { useCallback, useEffect, useMemo, useRef, useState } from "react";
import { zeroAddress } from "viem";
import { useAccount, useConfig } from "wagmi";
import { getAccount, signMessage } from "wagmi/actions";
var defaultStorage = new WebStorage();
function useXmtpIdentity({
  storage = defaultStorage,
  storageKey = "xmtpSigner"
} = {}) {
  const storageRef = useRef(storage);
  const [isLoading, setIsLoading] = useState(false);
  const [xmtpSigner, setXmtpSigner] = useState(null);
  const [xmtpClient, setXmtpClient] = useState(null);
  const config = useConfig();
  const connect = useConnectModal();
  const { address } = useAccount();
  const walletSigner = useMemo(
    () => address ? {
      getAddress() {
        return Promise.resolve(address);
      },
      signMessage(message) {
        return signMessage(config, {
          message: {
            raw: typeof message === "string" ? Buffer.from(message) : Buffer.from(message)
          }
        });
      }
    } : null,
    [address, config]
  );
  useEffect(() => {
    async function instantiateXmtpSignerAndClient() {
      const storedSigner = await storageRef.current.get(storageKey);
      if (!storedSigner) {
        return;
      }
      const signer = {
        walletAddress: storedSigner.walletAddress,
        keys: Buffer.from(storedSigner.keys, "hex")
      };
      const client = await Client.create(null, {
        privateKeyOverride: signer.keys
      });
      setXmtpSigner(signer);
      setXmtpClient(client);
    }
    instantiateXmtpSignerAndClient().catch((e) => {
      console.error(
        "@frames.js/render: Could not instantiate the XMTP signer and client",
        e
      );
    });
  }, [storageKey]);
  const logout = useCallback(async () => {
    await storageRef.current.delete(storageKey);
    setXmtpClient(null);
    setXmtpSigner(null);
  }, [storageKey]);
  const onSignerlessFramePress = useCallback(async () => {
    try {
      setIsLoading(true);
      if (!xmtpSigner) {
        if (!walletSigner) {
          connect.openConnectModal?.();
          return;
        }
        const keys = await Client.getKeys(walletSigner, {
          env: "dev",
          skipContactPublishing: true,
          persistConversations: false
        });
        const client = await Client.create(null, {
          privateKeyOverride: keys
        });
        const walletAddress = getAccount(config).address || zeroAddress;
        await storageRef.current.set(storageKey, () => ({
          walletAddress,
          keys: Buffer.from(keys).toString("hex")
        }));
        setXmtpSigner({
          keys,
          walletAddress
        });
        setXmtpClient(client);
      }
    } catch (error) {
      console.error("@frames.js/render: API Call failed", error);
    } finally {
      setIsLoading(false);
    }
  }, [config, walletSigner, xmtpSigner, connect, storageKey]);
  const signFrameAction = useCallback(
    async (actionContext) => {
      if (!xmtpClient) {
        throw new Error("No xmtp client");
      }
      if (!address) {
        throw new Error("No address");
      }
      const framesClient = new FramesClient(xmtpClient);
      const payload = await framesClient.signFrameAction({
        frameUrl: actionContext.url,
        inputText: actionContext.inputText,
        state: actionContext.state,
        buttonIndex: actionContext.buttonIndex,
        conversationTopic: actionContext.frameContext.conversationTopic,
        participantAccountAddresses: actionContext.frameContext.participantAccountAddresses,
        ...actionContext.frameContext.groupId ? { groupId: actionContext.frameContext.groupId } : {},
        ...actionContext.frameContext.groupSecret ? { groupSecret: actionContext.frameContext.groupSecret } : {},
        ...actionContext.type === "tx-data" || actionContext.type === "tx-post" ? { address: actionContext.address } : {},
        ...actionContext.type === "tx-post" ? { transactionId: actionContext.transactionId } : {}
      });
      const searchParams = new URLSearchParams({
        postType: actionContext.type === "tx-post" ? "post" : actionContext.frameButton.action,
        postUrl: actionContext.target ?? "",
        specification: "openframes"
      });
      return {
        body: payload,
        searchParams
      };
    },
    [address, xmtpClient]
  );
  return useMemo(
    () => ({
      signer: xmtpSigner,
      hasSigner: !!xmtpSigner?.keys,
      signFrameAction,
      isLoadingSigner: isLoading,
      onSignerlessFramePress,
      logout
    }),
    [isLoading, logout, onSignerlessFramePress, signFrameAction, xmtpSigner]
  );
}

export {
  useXmtpIdentity
};
