import { Reducer } from 'react';

type AsyncDispatch<TAction> = (action: TAction) => Promise<void>;
type StorageableReducer<TState, TAction> = [TState, AsyncDispatch<TAction>];
declare function usePersistedReducer<TState, TAction>(reducer: Reducer<TState, TAction>, 
/**
 * Used when initial state has not been loaded yet
 */
initialState: TState, 
/**
 * Called once when the component is mounted
 */
loadInitialState: () => Promise<TState>, 
/**
 * Called each time the dispatch is called
 */
saveState: (state: TState) => Promise<void>): StorageableReducer<TState, TAction>;

export { usePersistedReducer };
