"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/identity/storage/web-storage.ts
var web_storage_exports = {};
__export(web_storage_exports, {
  WebStorage: () => WebStorage
});
module.exports = __toCommonJS(web_storage_exports);
var localStorageUnavailableMessage = "WebStorage: localStorage is not available, consider implementing your own Storage class";
var WebStorage = class {
  /**
   * Keeps latest values in memory so we don't need to read them every time
   * we set or read the value.
   */
  values;
  watchers = {};
  constructor() {
    this.values = /* @__PURE__ */ new Map();
    this.watchers = {};
  }
  get = async (key) => {
    if (this.values.has(key)) {
      return Promise.resolve(this.values.get(key));
    }
    if (typeof localStorage !== "undefined") {
      const value = localStorage.getItem(key);
      if (!value) {
        return Promise.resolve(void 0);
      }
      const parsedValue = JSON.parse(value);
      this.values.set(key, parsedValue);
      return Promise.resolve(parsedValue);
    }
    console.warn(localStorageUnavailableMessage);
    return Promise.resolve(void 0);
  };
  set = async (key, setter) => {
    const currentValue = await this.get(key);
    const newValue = setter(currentValue);
    this.values.set(key, newValue);
    this.notifyChange(key, newValue);
    if (typeof localStorage !== "undefined") {
      localStorage.setItem(key, JSON.stringify(newValue));
      return;
    }
    console.warn(localStorageUnavailableMessage);
  };
  delete = (key) => {
    this.values.delete(key);
    this.notifyChange(key, void 0);
    if (typeof localStorage !== "undefined") {
      localStorage.removeItem(key);
      return Promise.resolve();
    }
    console.warn(localStorageUnavailableMessage);
    return Promise.resolve();
  };
  watch = (key, listener) => {
    this.watchers[key] ??= [];
    this.watchers[key]?.push(listener);
    this.get(key).then((value) => {
      listener(value);
    }).catch((e) => {
      console.error(
        `@frames.js/render: Failed to get value from storage: ${e}`
      );
    });
    return () => {
      const listeners = this.watchers[key];
      if (listeners) {
        this.watchers[key] = listeners.filter(
          (watcher) => watcher !== listener
        );
      }
    };
  };
  notifyChange = (key, value) => {
    this.watchers[key]?.forEach((listener) => {
      listener(value);
    });
  };
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  WebStorage
});
