export { VisibilityDetectionHook } from '../hooks/use-visibility-detection.js';

type StorageSetterFunction<TValue> = (value: TValue | undefined) => TValue;
type StorageValueWatcher<TValue> = (value: TValue | undefined) => void;
interface Storage {
    get: <T>(key: string) => Promise<T | undefined>;
    set: <T>(key: string, setter: StorageSetterFunction<T>) => Promise<void>;
    delete: (key: string) => Promise<void>;
    /**
     * Listener is called immediatelly after subscription so you get current value
     */
    watch: <T>(key: string, listener: StorageValueWatcher<T>) => () => void;
}
interface FrameContextManager<TFrameContext> {
    frameContext: TFrameContext;
    setFrameContext: (frameContext: TFrameContext) => Promise<void>;
    resetFrameContext: () => Promise<void>;
}

export type { FrameContextManager, Storage, StorageSetterFunction, StorageValueWatcher };
