"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/use-frame.tsx
var use_frame_exports = {};
__export(use_frame_exports, {
  useFrame: () => useFrame
});
module.exports = __toCommonJS(use_frame_exports);
var import_react2 = require("react");

// src/farcaster/frames.tsx
var import_core = require("@farcaster/core");
var import_viem = require("viem");
function isFarcasterFrameContext(frameContext) {
  return "castId" in frameContext;
}
async function unsignedFrameAction(actionContext) {
  const {
    frameButton,
    target,
    frameContext,
    url,
    buttonIndex,
    state,
    inputText
  } = actionContext;
  const searchParams = new URLSearchParams({
    postType: actionContext.type === "tx-post" ? "post" : frameButton.action,
    postUrl: target ?? ""
  });
  return Promise.resolve({
    searchParams,
    body: {
      untrustedData: {
        url,
        ...isFarcasterFrameContext(frameContext) ? {
          castId: {
            fid: frameContext.castId.fid,
            hash: frameContext.castId.hash
          },
          network: 1,
          timestamp: (0, import_core.getFarcasterTime)()._unsafeUnwrap()
        } : {
          timestamp: Date.now()
        },
        buttonIndex,
        state,
        inputText,
        address: actionContext.type === "tx-data" || actionContext.type === "tx-post" ? actionContext.address : void 0,
        transactionId: actionContext.type === "tx-post" ? actionContext.transactionId : void 0
      },
      trustedData: {
        messageBytes: Buffer.from("").toString("hex")
      }
    }
  });
}

// src/farcaster/attribution.ts
var import_viem2 = require("viem");

// src/use-frame-stack.ts
var import_react = require("react");
function computeDurationInSeconds(start, end) {
  return Number(((end.getTime() - start.getTime()) / 1e3).toFixed(2));
}
function isParseResult(result) {
  return typeof result === "object" && result !== null && "status" in result;
}
function framesStackReducer(state, action) {
  switch (action.action) {
    case "LOAD":
      return [action.item, ...state];
    case "DONE_REDIRECT": {
      const index = state.findIndex(
        (item) => item.timestamp === action.pendingItem.timestamp
      );
      if (index === -1) {
        return state;
      }
      state[index] = {
        ...action.pendingItem,
        ...action.item,
        status: "doneRedirect"
      };
      return state.slice();
    }
    case "DONE":
    case "REQUEST_ERROR": {
      const index = state.findIndex(
        (item) => item.timestamp === action.pendingItem.timestamp
      );
      if (index === -1) {
        return state;
      }
      state[index] = action.item;
      return state.slice();
    }
    case "RESET_INITIAL_FRAME": {
      const originalInitialFrame = state[0];
      const frame = isParseResult(action.resultOrFrame) ? action.resultOrFrame.frame : action.resultOrFrame;
      const shouldReset = !originalInitialFrame || originalInitialFrame.status === "done" && originalInitialFrame.frameResult.frame !== frame;
      if (shouldReset) {
        const frameResult = isParseResult(action.resultOrFrame) ? action.resultOrFrame : {
          status: "success",
          reports: {},
          frame: action.resultOrFrame
        };
        return [
          {
            request: {
              method: "GET",
              url: action.homeframeUrl ?? ""
            },
            url: action.homeframeUrl ?? "",
            requestDetails: {},
            response: new Response(JSON.stringify(frameResult), {
              status: 200,
              headers: { "Content-Type": "application/json" }
            }),
            responseStatus: 200,
            timestamp: /* @__PURE__ */ new Date(),
            speed: 0,
            frameResult,
            status: "done",
            // @todo should this be result or frame?
            responseBody: frameResult
          }
        ];
      }
      return state;
    }
    case "CLEAR":
      return [];
    default:
      return state;
  }
}
function useFrameStack({
  initialFrame,
  initialFrameUrl
}) {
  const [stack, dispatch] = (0, import_react.useReducer)(
    framesStackReducer,
    [initialFrame, initialFrameUrl],
    ([frame, frameUrl]) => {
      if (frame) {
        const frameResult = isParseResult(frame) ? frame : {
          reports: {},
          frame,
          status: "success"
        };
        return [
          {
            response: new Response(JSON.stringify(frameResult), {
              status: 200,
              headers: { "Content-Type": "application/json" }
            }),
            responseStatus: 200,
            responseBody: frameResult,
            timestamp: /* @__PURE__ */ new Date(),
            requestDetails: {},
            request: {
              method: "GET",
              url: frameUrl ?? ""
            },
            speed: 0,
            frameResult,
            status: "done",
            url: frameUrl ?? ""
          }
        ];
      } else if (frameUrl) {
        return [
          {
            method: "GET",
            request: {
              method: "GET",
              url: frameUrl
            },
            url: frameUrl,
            requestDetails: {},
            timestamp: /* @__PURE__ */ new Date(),
            status: "pending"
          }
        ];
      }
      return [];
    }
  );
  const api = (0, import_react.useMemo)(() => {
    return {
      clear() {
        dispatch({
          action: "CLEAR"
        });
      },
      createGetPendingItem(arg) {
        const item = {
          method: "GET",
          request: arg.request,
          requestDetails: {},
          url: arg.request.url,
          timestamp: /* @__PURE__ */ new Date(),
          status: "pending"
        };
        dispatch({
          action: "LOAD",
          item
        });
        return item;
      },
      createPostPendingItem(arg) {
        const item = {
          method: "POST",
          request: arg.request,
          requestDetails: {
            body: arg.action.body,
            searchParams: arg.action.searchParams
          },
          url: arg.action.searchParams.get("postUrl") ?? "missing postUrl",
          timestamp: arg.startTime ?? /* @__PURE__ */ new Date(),
          status: "pending"
        };
        dispatch({
          action: "LOAD",
          item
        });
        return item;
      },
      createCastOrComposerActionPendingItem(arg) {
        return {
          method: "POST",
          requestDetails: {
            body: arg.action.body,
            searchParams: arg.action.searchParams
          },
          request: arg.request,
          status: "pending",
          timestamp: /* @__PURE__ */ new Date(),
          url: arg.action.searchParams.get("postUrl") ?? "missing postUrl"
        };
      },
      markCastFrameAsDone() {
      },
      markCastMessageAsDone(arg) {
        dispatch({
          action: "LOAD",
          item: arg.pendingItem
        });
        dispatch({
          action: "DONE",
          pendingItem: arg.pendingItem,
          item: {
            ...arg.pendingItem,
            status: "message",
            message: arg.responseData.message,
            response: arg.response.clone(),
            responseBody: arg.responseData,
            responseStatus: arg.response.status,
            speed: computeDurationInSeconds(
              arg.pendingItem.timestamp,
              arg.endTime
            ),
            type: "info"
          }
        });
      },
      markComposerFormActionAsDone() {
      },
      markAsDone(arg) {
        dispatch({
          action: "DONE",
          pendingItem: arg.pendingItem,
          item: {
            ...arg.pendingItem,
            status: "done",
            frameResult: arg.frameResult,
            speed: computeDurationInSeconds(
              arg.pendingItem.timestamp,
              arg.endTime
            ),
            response: arg.response.clone(),
            responseStatus: arg.response.status,
            responseBody: arg.frameResult
          }
        });
      },
      markAsDoneWithErrorMessage(arg) {
        dispatch({
          action: "DONE",
          pendingItem: arg.pendingItem,
          item: {
            ...arg.pendingItem,
            responseStatus: arg.response.status,
            response: arg.response.clone(),
            speed: computeDurationInSeconds(
              arg.pendingItem.timestamp,
              arg.endTime
            ),
            status: "message",
            type: "error",
            message: arg.responseData.message,
            responseBody: arg.responseData
          }
        });
      },
      markAsDoneWithRedirect(arg) {
        dispatch({
          action: "DONE_REDIRECT",
          pendingItem: arg.pendingItem,
          item: {
            ...arg.pendingItem,
            location: arg.location,
            response: arg.response.clone(),
            responseBody: arg.responseBody,
            responseStatus: arg.response.status,
            status: "doneRedirect",
            speed: computeDurationInSeconds(
              arg.pendingItem.timestamp,
              arg.endTime
            )
          }
        });
      },
      markAsFailed(arg) {
        dispatch({
          action: "REQUEST_ERROR",
          pendingItem: arg.pendingItem,
          item: {
            request: arg.pendingItem.request,
            requestDetails: arg.pendingItem.requestDetails,
            timestamp: arg.pendingItem.timestamp,
            url: arg.pendingItem.url,
            response: arg.response?.clone() ?? null,
            responseStatus: arg.responseStatus,
            requestError: arg.requestError,
            speed: computeDurationInSeconds(
              arg.pendingItem.timestamp,
              arg.endTime
            ),
            status: "requestError",
            responseBody: arg.responseBody
          }
        });
      },
      markAsFailedWithRequestError(arg) {
        dispatch({
          action: "REQUEST_ERROR",
          pendingItem: arg.pendingItem,
          item: {
            ...arg.pendingItem,
            status: "requestError",
            requestError: arg.error,
            response: arg.response.clone(),
            responseStatus: arg.response.status,
            responseBody: arg.responseBody,
            speed: computeDurationInSeconds(
              arg.pendingItem.timestamp,
              arg.endTime
            )
          }
        });
      }
    };
  }, [dispatch]);
  return [stack, dispatch, api];
}

// src/use-fetch-frame.ts
var import_viem3 = require("viem");

// src/errors.ts
var TransactionDataTargetMalformedError = class extends Error {
  constructor(response) {
    super("Malformed transaction target data from the server");
    this.response = response;
  }
};
var TransactionDataErrorResponseError = class extends Error {
  constructor(response) {
    super("Non OK transaction data response from the server");
    this.response = response;
  }
};
var TransactionHandlerDidNotReturnTransactionIdError = class extends Error {
  constructor() {
    super("onTransaction() handler did not return a transaction id");
  }
};
var SignatureHandlerDidNotReturnTransactionIdError = class extends Error {
  constructor() {
    super("onSignature() handler did not return a transaction id");
  }
};
var CastActionUnexpectedResponseError = class extends Error {
  constructor() {
    super("Unexpected cast action response from the server");
  }
};
var ComposerActionUnexpectedResponseError = class extends Error {
  constructor() {
    super("Unexpected composer action response from the server");
  }
};

// src/helpers.ts
async function tryCallAsync(promiseFn) {
  try {
    return promiseFn().catch((e) => {
      if (e instanceof Error) {
        return e;
      }
      return new TypeError("Unexpected error, check the console for details");
    });
  } catch (e) {
    return new TypeError("Unexpected error, check the console for details");
  }
}
function tryCall(fn) {
  try {
    return fn();
  } catch (e) {
    if (e instanceof Error) {
      return e;
    }
    return new TypeError("Unexpected error, check the console for details");
  }
}

// src/use-fetch-frame.ts
function isErrorMessageResponse(response) {
  return typeof response === "object" && response !== null && "message" in response && typeof response.message === "string";
}
function isComposerFormActionResponse(response) {
  return typeof response === "object" && response !== null && "type" in response && response.type === "form";
}
function isCastActionFrameResponse(response) {
  return typeof response === "object" && response !== null && "type" in response && response.type === "frame";
}
function isCastMessageResponse(response) {
  return typeof response === "object" && response !== null && "message" in response && typeof response.message === "string";
}
function defaultErrorHandler(error) {
  console.error(error);
}
function useFetchFrame({
  stackAPI,
  stackDispatch,
  specification,
  frameActionProxy,
  frameGetProxy,
  extraButtonRequestPayload,
  signFrameAction,
  onTransaction,
  transactionDataSuffix,
  onSignature,
  onError = defaultErrorHandler,
  fetchFn,
  onRedirect,
  onComposerFormAction,
  onTransactionDataError,
  onTransactionDataStart,
  onTransactionDataSuccess,
  onTransactionError,
  onTransactionStart,
  onTransactionSuccess,
  onSignatureError,
  onSignatureStart,
  onSignatureSuccess,
  onTransactionProcessingError,
  onTransactionProcessingStart,
  onTransactionProcessingSuccess
}) {
  async function handleFailedResponse({
    response,
    endTime,
    frameStackPendingItem,
    onError: onErrorInternal
  }) {
    if (response.ok) {
      throw new TypeError(
        "handleFailedResponse called with a successful response"
      );
    }
    const responseBody = await getResponseBody(response);
    if (response.status >= 400 && response.status < 500) {
      if (frameStackPendingItem.method === "POST" && isErrorMessageResponse(responseBody)) {
        stackAPI.markAsDoneWithErrorMessage({
          pendingItem: frameStackPendingItem,
          endTime,
          response,
          responseData: responseBody
        });
        const error = new Error(responseBody.message);
        tryCall(() => {
          onError(error);
        });
        tryCall(() => onErrorInternal?.(error));
        return;
      }
    }
    const requestError = new Error(
      `The server returned an error but it does not contain message property. Status code: ${response.status}`
    );
    stackAPI.markAsFailed({
      endTime,
      pendingItem: frameStackPendingItem,
      requestError,
      responseStatus: response.status,
      response,
      responseBody
    });
    tryCall(() => {
      onError(requestError);
    });
    tryCall(() => onErrorInternal?.(requestError));
  }
  async function fetchGETRequest(request, shouldClear) {
    if (shouldClear) {
      stackDispatch({ action: "CLEAR" });
    }
    const frameStackPendingItem = stackAPI.createGetPendingItem({ request });
    const response = await fetchProxied({
      proxyUrl: frameGetProxy,
      fetchFn,
      specification,
      url: request.url
    });
    const endTime = /* @__PURE__ */ new Date();
    if (response instanceof Response) {
      if (!response.ok) {
        await handleFailedResponse({
          response,
          endTime,
          frameStackPendingItem
        });
        return;
      }
      const frameResult = await response.clone().json();
      stackAPI.markAsDone({
        pendingItem: frameStackPendingItem,
        endTime,
        frameResult,
        response
      });
      return;
    }
    stackAPI.markAsFailed({
      pendingItem: frameStackPendingItem,
      endTime,
      requestError: response,
      response: null,
      responseBody: "none",
      responseStatus: 500
    });
    tryCall(() => {
      onError(response);
    });
  }
  async function fetchPOSTRequest(request, options) {
    let pendingItem;
    if (options?.shouldClear) {
      stackDispatch({ action: "CLEAR" });
    }
    const signedDataOrError = await signAndGetFrameActionBodyPayload({
      signerStateActionContext: request.signerStateActionContext,
      signFrameAction
    });
    if (signedDataOrError instanceof Error) {
      if (options?.preflightRequest) {
        stackAPI.markAsFailed({
          pendingItem: options.preflightRequest.pendingFrameStackItem,
          endTime: /* @__PURE__ */ new Date(),
          requestError: signedDataOrError,
          response: null,
          // there is no response because didn't even got to request
          responseBody: "none",
          responseStatus: 500
        });
      }
      tryCall(() => {
        onError(signedDataOrError);
      });
      tryCall(() => options?.onError?.(signedDataOrError));
      return;
    }
    if (!options?.preflightRequest) {
      pendingItem = stackAPI.createPostPendingItem({
        action: signedDataOrError,
        request
      });
    } else {
      pendingItem = options.preflightRequest.pendingFrameStackItem;
    }
    const response = await fetchProxied({
      proxyUrl: frameActionProxy,
      specification,
      fetchFn,
      frameAction: signedDataOrError,
      extraRequestPayload: extraButtonRequestPayload
    });
    const endTime = /* @__PURE__ */ new Date();
    async function handleRedirect({
      response: res,
      currentPendingItem,
      onError: onErrorInternal,
      onSuccess: onSuccessInternal
    }) {
      try {
        let location = res.headers.get("location");
        if (!location) {
          const responseData = await res.clone().json();
          if (responseData && typeof responseData === "object" && "location" in responseData && typeof responseData.location === "string") {
            location = responseData.location;
          }
        }
        if (!location) {
          throw new Error(
            `Response data does not contain 'location' key and no 'location' header is found.`
          );
        }
        const locationUrl = new URL(location);
        if (locationUrl.protocol !== "http:" && locationUrl.protocol !== "https:") {
          throw new Error(
            `Redirect location ${location} is not a valid HTTP or HTTPS URL.`
          );
        }
        tryCall(() => {
          onRedirect(locationUrl);
        });
        tryCall(() => onSuccessInternal?.());
        stackAPI.markAsDoneWithRedirect({
          pendingItem: currentPendingItem,
          endTime,
          location,
          response: res.clone(),
          responseBody: await res.clone().text()
        });
      } catch (e) {
        const error = e instanceof Error ? e : new Error(
          "Response body must be a json with 'location' property or response 'Location' header must contain fully qualified URL."
        );
        stackAPI.markAsFailedWithRequestError({
          pendingItem: currentPendingItem,
          error,
          response: res,
          endTime,
          responseBody: await res.clone().text()
        });
        tryCall(() => onErrorInternal?.(error));
        tryCall(() => {
          onError(error);
        });
      }
    }
    if (response instanceof Response) {
      if (response.status === 302) {
        await handleRedirect({
          response,
          currentPendingItem: pendingItem,
          onError: options?.onError,
          onSuccess: options?.onSuccess
        });
        return;
      }
      if (!response.ok) {
        await handleFailedResponse({
          response,
          endTime,
          frameStackPendingItem: pendingItem,
          onError: options?.onError
        });
        return;
      }
      const responseData = await tryCall(
        () => response.clone().json()
      );
      if (responseData instanceof Error) {
        stackAPI.markAsFailed({
          endTime,
          pendingItem,
          requestError: responseData,
          response,
          responseBody: "none",
          responseStatus: 500
        });
        tryCall(() => {
          onError(responseData);
        });
        tryCall(() => options?.onError?.(responseData));
        return;
      }
      if (!isParseResult(responseData)) {
        const error = new Error("The server returned an unexpected response.");
        stackAPI.markAsFailed({
          endTime,
          pendingItem,
          requestError: error,
          response,
          responseBody: responseData,
          responseStatus: 500
        });
        tryCall(() => {
          onError(error);
        });
        tryCall(() => options?.onError?.(error));
        return;
      }
      stackAPI.markAsDone({
        endTime,
        frameResult: responseData,
        pendingItem,
        response
      });
      tryCall(() => options?.onSuccess?.());
      return;
    }
    stackAPI.markAsFailed({
      endTime,
      pendingItem,
      requestError: response,
      response: new Response(response.message, {
        status: 500,
        headers: { "Content-Type": "text/plain" }
      }),
      responseBody: "none",
      responseStatus: 500
    });
    tryCall(() => {
      onError(response);
    });
    tryCall(() => options?.onError?.(response));
  }
  async function fetchTransactionRequest(request, shouldClear) {
    if ("source" in request) {
      throw new Error(
        "Invalid request, transaction should be invoked only from a Frame. It was probably invoked from cast or composer action."
      );
    }
    const button = request.frameButton;
    const sourceFrame = request.sourceFrame;
    if (button.action !== "tx") {
      throw new Error("Invalid frame button action, tx expected");
    }
    if (request.signerStateActionContext.type !== "tx-data") {
      throw new Error(
        "Invalid signer state action context type, tx-data expected"
      );
    }
    if (shouldClear) {
      stackDispatch({ action: "CLEAR" });
    }
    tryCall(() => onTransactionDataStart?.({ button }));
    const signedTransactionDataActionOrError = await tryCallAsync(
      () => signFrameAction({
        actionContext: request.signerStateActionContext,
        // for transaction data we always use signer, so skip signing is false here
        forceRealSigner: true
      })
    );
    if (signedTransactionDataActionOrError instanceof Error) {
      tryCall(() => {
        onError(signedTransactionDataActionOrError);
      });
      tryCall(
        () => onTransactionDataError?.(signedTransactionDataActionOrError)
      );
      return;
    }
    signedTransactionDataActionOrError.searchParams.set(
      "specification",
      specification
    );
    const transactionDataStartTime = /* @__PURE__ */ new Date();
    const transactionDataResponse = await fetchProxied({
      proxyUrl: frameActionProxy,
      frameAction: signedTransactionDataActionOrError,
      fetchFn,
      specification,
      extraRequestPayload: extraButtonRequestPayload
    });
    const transactionDataEndTime = /* @__PURE__ */ new Date();
    if (transactionDataResponse instanceof Error) {
      const pendingItem2 = stackAPI.createPostPendingItem({
        action: signedTransactionDataActionOrError,
        request,
        startTime: transactionDataStartTime
      });
      stackAPI.markAsFailed({
        endTime: transactionDataEndTime,
        pendingItem: pendingItem2,
        requestError: transactionDataResponse,
        response: null,
        responseBody: "none",
        responseStatus: 500
      });
      tryCall(() => onTransactionDataError?.(transactionDataResponse));
      tryCall(() => {
        onError(transactionDataResponse);
      });
      return;
    }
    if (!transactionDataResponse.ok) {
      const pendingItem2 = stackAPI.createPostPendingItem({
        action: signedTransactionDataActionOrError,
        request,
        startTime: transactionDataStartTime
      });
      await handleFailedResponse({
        response: transactionDataResponse,
        endTime: transactionDataEndTime,
        frameStackPendingItem: pendingItem2
      });
      tryCall(
        () => onTransactionDataError?.(
          new TransactionDataErrorResponseError(transactionDataResponse.clone())
        )
      );
      return;
    }
    function isTransactionTargetResponse(response) {
      return typeof response === "object" && response !== null && "method" in response;
    }
    const transactionData = await tryCallAsync(
      () => transactionDataResponse.clone().json()
    );
    if (!isTransactionTargetResponse(transactionData)) {
      const pendingItem2 = stackAPI.createPostPendingItem({
        action: signedTransactionDataActionOrError,
        request,
        startTime: transactionDataStartTime
      });
      const error = new TransactionDataTargetMalformedError(
        transactionDataResponse.clone()
      );
      stackAPI.markAsFailed({
        endTime: transactionDataEndTime,
        pendingItem: pendingItem2,
        requestError: error,
        response: transactionDataResponse,
        responseBody: transactionData,
        responseStatus: 500
      });
      tryCall(() => onTransactionDataError?.(error));
      tryCall(() => {
        onError(error);
      });
      return;
    }
    tryCall(
      () => onTransactionDataSuccess?.({ button, data: transactionData })
    );
    let transactionIdOrError;
    if (transactionData.method === "eth_sendTransaction") {
      if (transactionData.params.data && transactionData.attribution !== false && transactionDataSuffix && // Has a function signature
      (0, import_viem3.hexToBytes)(transactionData.params.data).length > 4) {
        transactionData.params.data = transactionData.params.data + transactionDataSuffix.slice(2);
      }
      tryCall(() => onTransactionStart?.({ button, data: transactionData }));
      transactionIdOrError = await tryCallAsync(
        () => onTransaction({
          frame: sourceFrame,
          frameButton: request.frameButton,
          transactionData
        }).then((transactionId) => {
          if (!transactionId) {
            return new TransactionHandlerDidNotReturnTransactionIdError();
          }
          return transactionId;
        })
      );
      if (!(transactionIdOrError instanceof Error)) {
        tryCall(() => onTransactionSuccess?.({ button }));
      } else {
        tryCall(() => onTransactionError?.(transactionIdOrError));
      }
    } else {
      tryCall(() => onSignatureStart?.({ button, data: transactionData }));
      transactionIdOrError = await tryCallAsync(
        () => onSignature({
          frame: sourceFrame,
          frameButton: request.frameButton,
          signatureData: transactionData
        }).then((signatureHash) => {
          if (!signatureHash) {
            return new SignatureHandlerDidNotReturnTransactionIdError();
          }
          return signatureHash;
        })
      );
      if (!(transactionIdOrError instanceof Error)) {
        tryCall(() => onSignatureSuccess?.({ button }));
      } else {
        tryCall(() => onSignatureError?.(transactionIdOrError));
      }
    }
    if (transactionIdOrError instanceof Error) {
      const pendingItem2 = stackAPI.createPostPendingItem({
        action: signedTransactionDataActionOrError,
        request
      });
      stackAPI.markAsFailed({
        pendingItem: pendingItem2,
        endTime: /* @__PURE__ */ new Date(),
        requestError: transactionIdOrError,
        response: null,
        responseBody: "none",
        responseStatus: 500
      });
      tryCall(() => {
        onError(transactionIdOrError);
      });
      return;
    }
    tryCall(
      () => onTransactionProcessingStart?.({
        button,
        transactionId: transactionIdOrError
      })
    );
    const startTime = /* @__PURE__ */ new Date();
    const pendingItem = stackAPI.createPostPendingItem({
      action: signedTransactionDataActionOrError,
      request
    });
    await fetchPOSTRequest(
      {
        ...request,
        signerStateActionContext: {
          ...request.signerStateActionContext,
          type: "tx-post",
          // include transactionId in payload
          transactionId: transactionIdOrError,
          // override target so the the request is sent to proper endpoint
          target: button.post_url || sourceFrame.postUrl || button.target
        }
      },
      {
        // we are continuing with the same pending item
        preflightRequest: {
          pendingFrameStackItem: pendingItem,
          startTime
        },
        onError(error) {
          tryCall(() => onTransactionProcessingError?.(error));
        },
        onSuccess() {
          tryCall(
            () => onTransactionProcessingSuccess?.({
              button,
              transactionId: transactionIdOrError
            })
          );
        }
      }
    );
  }
  async function fetchCastActionRequest(request, shouldClear = false) {
    const frameButton = {
      action: "post",
      label: request.action.name,
      target: request.action.action.postUrl || request.action.url
    };
    const signerStateActionContext = {
      ...request.signerStateActionContext,
      type: "default",
      frameButton
    };
    const signedDataOrError = await signAndGetFrameActionBodyPayload({
      signerStateActionContext,
      signFrameAction
    });
    if (shouldClear) {
      stackAPI.clear();
    }
    if (signedDataOrError instanceof Error) {
      tryCall(() => {
        onError(signedDataOrError);
      });
      throw signedDataOrError;
    }
    const pendingItem = stackAPI.createCastOrComposerActionPendingItem({
      action: signedDataOrError,
      request: {
        ...request,
        frameButton,
        signerStateActionContext,
        method: "POST",
        source: "cast-action",
        sourceFrame: void 0
      }
    });
    const actionResponseOrError = await fetchProxied({
      fetchFn,
      proxyUrl: frameActionProxy,
      specification,
      frameAction: signedDataOrError,
      extraRequestPayload: extraButtonRequestPayload
    });
    if (actionResponseOrError instanceof Error) {
      tryCall(() => {
        onError(actionResponseOrError);
      });
      throw actionResponseOrError;
    }
    try {
      const endTime = /* @__PURE__ */ new Date();
      if (!actionResponseOrError.ok) {
        await handleFailedResponse({
          response: actionResponseOrError,
          endTime,
          frameStackPendingItem: pendingItem
        });
        return;
      }
      const actionResponse = await actionResponseOrError.clone().json();
      if (isCastMessageResponse(actionResponse)) {
        stackAPI.markCastMessageAsDone({
          pendingItem,
          endTime,
          response: actionResponseOrError,
          responseData: actionResponse
        });
        return;
      }
      if (isCastActionFrameResponse(actionResponse)) {
        stackAPI.markCastFrameAsDone({ pendingItem, endTime });
        await fetchPOSTRequest({
          sourceFrame: void 0,
          frameButton: {
            action: "post",
            label: "action",
            target: actionResponse.frameUrl
          },
          isDangerousSkipSigning: request.isDangerousSkipSigning,
          method: "POST",
          source: "cast-action",
          signerStateActionContext: {
            ...request.signerStateActionContext,
            type: "default",
            buttonIndex: 1,
            frameButton: {
              action: "post",
              label: "action",
              target: actionResponse.frameUrl
            },
            target: actionResponse.frameUrl
          }
        });
        return;
      }
      throw new CastActionUnexpectedResponseError();
    } catch (e) {
      let error;
      if (!(e instanceof CastActionUnexpectedResponseError)) {
        console.error(`Unexpected response from the server`, e);
        error = e instanceof Error ? e : new Error("Unexpected error");
      } else {
        error = e;
      }
      tryCall(() => {
        onError(error);
      });
      throw error;
    }
  }
  async function fetchComposerActionRequest(request, shouldClear = false) {
    const frameButton = {
      action: "post",
      label: request.action.name,
      target: request.action.url
    };
    const signerStateActionContext = {
      ...request.signerStateActionContext,
      type: "default",
      frameButton,
      state: encodeURIComponent(
        JSON.stringify({
          cast: request.composerActionState
        })
      )
    };
    const signedDataOrError = await signAndGetFrameActionBodyPayload({
      signerStateActionContext,
      signFrameAction
    });
    if (shouldClear) {
      stackAPI.clear();
    }
    if (signedDataOrError instanceof Error) {
      tryCall(() => {
        onError(signedDataOrError);
      });
      throw signedDataOrError;
    }
    const pendingItem = stackAPI.createCastOrComposerActionPendingItem({
      action: signedDataOrError,
      request: {
        ...request,
        frameButton,
        signerStateActionContext,
        method: "POST",
        source: "composer-action",
        sourceFrame: void 0
      }
    });
    const actionResponseOrError = await fetchProxied({
      fetchFn,
      proxyUrl: frameActionProxy,
      specification,
      frameAction: signedDataOrError,
      extraRequestPayload: extraButtonRequestPayload
    });
    if (actionResponseOrError instanceof Error) {
      tryCall(() => {
        onError(actionResponseOrError);
      });
      throw actionResponseOrError;
    }
    try {
      const endTime = /* @__PURE__ */ new Date();
      if (!actionResponseOrError.ok) {
        await handleFailedResponse({
          response: actionResponseOrError,
          endTime,
          frameStackPendingItem: pendingItem
        });
        return;
      }
      const actionResponse = await actionResponseOrError.clone().json();
      if (!isComposerFormActionResponse(actionResponse)) {
        throw new ComposerActionUnexpectedResponseError();
      }
      stackAPI.markComposerFormActionAsDone({ pendingItem, endTime });
      await onComposerFormAction({
        form: actionResponse,
        cast: {
          embeds: [],
          text: "Cast text"
        }
      });
    } catch (e) {
      let error;
      if (!(e instanceof ComposerActionUnexpectedResponseError)) {
        console.error(`Unexpected response from the server`, e);
        error = e instanceof Error ? e : new Error("Unexpected error");
      } else {
        error = e;
      }
      tryCall(() => {
        onError(error);
      });
      throw error;
    }
  }
  return (request, shouldClear = false) => {
    if (request.method === "GET") {
      return fetchGETRequest(request, shouldClear);
    }
    if (request.method === "CAST_ACTION") {
      return fetchCastActionRequest(request, shouldClear);
    }
    if (request.method === "COMPOSER_ACTION") {
      return fetchComposerActionRequest(request, shouldClear);
    }
    if (request.frameButton.action === "tx") {
      return fetchTransactionRequest(request, shouldClear);
    }
    return fetchPOSTRequest(request, {
      shouldClear
    });
  };
}
function proxyUrlAndSearchParamsToUrl(proxyUrl, ...searchParams) {
  const temporaryDomain = "temporary-for-parsing-purposes.tld";
  const parsedProxyUrl = new URL(proxyUrl, `http://${temporaryDomain}`);
  searchParams.forEach((params) => {
    params.forEach((value, key) => {
      parsedProxyUrl.searchParams.set(key, value);
    });
  });
  return parsedProxyUrl.hostname === temporaryDomain ? `${parsedProxyUrl.pathname}${parsedProxyUrl.search}` : parsedProxyUrl.toString();
}
async function fetchProxied(params) {
  const searchParams = new URLSearchParams({
    specification: params.specification
  });
  if ("frameAction" in params) {
    const proxyUrl2 = proxyUrlAndSearchParamsToUrl(
      params.proxyUrl,
      searchParams,
      params.frameAction.searchParams
    );
    return tryCallAsync(
      () => params.fetchFn(proxyUrl2, {
        method: "POST",
        headers: {
          "Content-Type": "application/json"
        },
        body: JSON.stringify({
          ...params.extraRequestPayload,
          ...params.frameAction.body
        })
      })
    );
  }
  searchParams.set("url", params.url);
  const proxyUrl = proxyUrlAndSearchParamsToUrl(params.proxyUrl, searchParams);
  return tryCallAsync(() => params.fetchFn(proxyUrl, { method: "GET" }));
}
function getResponseBody(response) {
  if (response.headers.get("content-type")?.includes("/json")) {
    return response.clone().json();
  }
  return response.clone().text();
}
async function signAndGetFrameActionBodyPayload({
  signerStateActionContext,
  signFrameAction
}) {
  const { address: _, ...requiredFrameContext } = signerStateActionContext.frameContext;
  return tryCallAsync(
    () => signFrameAction({
      actionContext: {
        ...signerStateActionContext,
        frameContext: requiredFrameContext
      }
    })
  );
}

// src/use-frame.tsx
function onMintFallback({ target }) {
  console.log("Please provide your own onMint function to useFrame() hook.");
  const message = `Mint requested: ${target}`;
  if (typeof window !== "undefined") {
    window.alert(message);
  } else {
    console.log(message);
  }
}
function onConnectWalletFallback() {
  throw new Error(
    "Please implement this function in order to use transactions"
  );
}
async function onTransactionFallback({
  transactionData
}) {
  console.log(
    "Please provide your own onTransaction function to useFrame() hook."
  );
  const message = `Requesting a transaction on chain with ID ${transactionData.chainId} with the following params: ${JSON.stringify(
    transactionData.params,
    null,
    2
  )}`;
  if (typeof window !== "undefined") {
    window.alert(message);
  } else {
    console.log(message);
  }
  return null;
}
async function onSignatureFallback({
  signatureData
}) {
  console.log(
    "Please provide your own onSignature function to useFrame() hook."
  );
  const message = `Requesting a signature on chain with ID ${signatureData.chainId} with the following params: ${JSON.stringify(
    signatureData.params,
    null,
    2
  )}`;
  if (typeof window !== "undefined") {
    window.alert(message);
  } else {
    console.log(message);
  }
  return null;
}
function handleRedirectFallback(location) {
  console.log(
    "Please provide your own onRedirect function to useFetchFrame() hook."
  );
  const message = `You are about to be redirected to ${location.toString()}`;
  if (typeof window !== "undefined") {
    if (window.confirm(message)) {
      window.open(location, "_blank")?.focus();
    }
  } else {
    console.log(message);
  }
}
function handleLinkButtonClickFallback(button) {
  console.log(
    "Please provide your own onLinkButtonClick function to useFrame() hook."
  );
  if (typeof window !== "undefined") {
    if (window.confirm(`You are about to be redirected to ${button.target}`)) {
      parent.window.open(button.target, "_blank");
    }
  } else {
    console.log(`Link button with target ${button.target} clicked.`);
  }
}
function defaultComposerFormActionHandler() {
  throw new Error('Please implement your own "onComposerFormAction" handler');
}
function validateLinkButtonTarget(target) {
  const locationUrl = new URL(target);
  if (locationUrl.protocol !== "http:" && locationUrl.protocol !== "https:") {
    throw new Error(
      `Redirect location ${locationUrl.toString()} is not a valid HTTP or HTTPS URL.`
    );
  }
  return true;
}
function useFrame({
  homeframeUrl,
  frameContext,
  dangerousSkipSigning,
  onMint = onMintFallback,
  onTransaction = onTransactionFallback,
  transactionDataSuffix,
  onConnectWallet = onConnectWalletFallback,
  onSignature = onSignatureFallback,
  connectedAddress,
  signerState,
  frame,
  /** Ex: /frames */
  frameActionProxy,
  /** Ex: /frames */
  frameGetProxy,
  extraButtonRequestPayload,
  specification = "farcaster",
  onError,
  onLinkButtonClick = handleLinkButtonClickFallback,
  onRedirect = handleRedirectFallback,
  fetchFn = (...args) => fetch(...args),
  onComposerFormAction = defaultComposerFormActionHandler,
  onTransactionDataError,
  onTransactionDataStart,
  onTransactionDataSuccess,
  onTransactionError,
  onTransactionProcessingError,
  onTransactionProcessingStart,
  onTransactionProcessingSuccess,
  onTransactionStart,
  onTransactionSuccess
}) {
  const [inputText, setInputText] = (0, import_react2.useState)("");
  const [framesStack, dispatch, stackAPI] = useFrameStack({
    initialFrame: frame,
    initialFrameUrl: homeframeUrl
  });
  const fetchFrame = useFetchFrame({
    stackAPI,
    frameActionProxy,
    frameGetProxy,
    onTransaction,
    transactionDataSuffix,
    onSignature,
    signFrameAction({ actionContext, forceRealSigner }) {
      return dangerousSkipSigning && !forceRealSigner ? unsignedFrameAction(actionContext) : signerState.signFrameAction(actionContext);
    },
    specification,
    stackDispatch: dispatch,
    extraButtonRequestPayload,
    onError,
    fetchFn,
    onRedirect,
    onComposerFormAction,
    onTransactionDataError,
    onTransactionDataStart,
    onTransactionDataSuccess,
    onTransactionError,
    onTransactionProcessingError,
    onTransactionProcessingStart,
    onTransactionProcessingSuccess,
    onTransactionStart,
    onTransactionSuccess
  });
  const fetchFrameRef = (0, import_react2.useRef)(fetchFrame);
  fetchFrameRef.current = fetchFrame;
  const onErrorRef = (0, import_react2.useRef)(onError);
  onErrorRef.current = onError;
  (0, import_react2.useEffect)(() => {
    if (!frame && homeframeUrl) {
      fetchFrameRef.current(
        {
          url: homeframeUrl,
          method: "GET"
        },
        // tell the fetchFrame function to clear the stack because this is called only on initial render
        // and there could potentially be a pending object returned from SSR
        true
      ).catch((e) => {
        console.error(e);
      });
    } else if (frame) {
      dispatch({
        action: "RESET_INITIAL_FRAME",
        resultOrFrame: frame,
        homeframeUrl
      });
    }
  }, [frame, homeframeUrl, dispatch]);
  const onPostButton = (0, import_react2.useCallback)(
    async function onPostButton2({
      currentFrame,
      buttonIndex,
      postInputText,
      frameButton,
      target,
      state,
      fetchFrameOverride
    }) {
      if (!dangerousSkipSigning && !signerState.hasSigner) {
        const error = new Error("Missing signer");
        onErrorRef.current?.(error);
        console.error(`@frames.js/render: ${error.message}`);
        return;
      }
      if (!homeframeUrl) {
        const error = new Error("Missing homeframeUrl");
        onErrorRef.current?.(error);
        console.error(`@frames.js/render: ${error.message}`);
        return;
      }
      const _fetchFrame = fetchFrameOverride ?? fetchFrame;
      await _fetchFrame({
        frameButton,
        isDangerousSkipSigning: dangerousSkipSigning ?? false,
        method: "POST",
        signerStateActionContext: {
          inputText: postInputText,
          signer: signerState.signer ?? null,
          frameContext,
          url: homeframeUrl,
          target,
          frameButton,
          buttonIndex,
          state
        },
        sourceFrame: currentFrame
      });
    },
    [
      dangerousSkipSigning,
      fetchFrame,
      frameContext,
      homeframeUrl,
      signerState.hasSigner,
      signerState.signer
    ]
  );
  const onConnectWalletRef = (0, import_react2.useRef)(onConnectWallet);
  onConnectWalletRef.current = onConnectWallet;
  const onTransactionButton = (0, import_react2.useCallback)(
    async function onTransactionButton2({
      currentFrame,
      buttonIndex,
      postInputText,
      frameButton
    }) {
      if (!dangerousSkipSigning && !signerState.hasSigner) {
        const error = new Error("Missing signer");
        onErrorRef.current?.(error);
        console.error(`@frames.js/render: ${error.message}`);
        return;
      }
      if (!homeframeUrl) {
        const error = new Error("Missing homeframeUrl");
        onErrorRef.current?.(error);
        console.error(`@frames.js/render: ${error.message}`);
        return;
      }
      if (!connectedAddress) {
        try {
          onConnectWalletRef.current();
        } catch (e) {
          onErrorRef.current?.(e instanceof Error ? e : new Error(String(e)));
          console.error(`@frames.js/render: ${String(e)}`);
        }
        return;
      }
      await fetchFrame({
        frameButton,
        isDangerousSkipSigning: dangerousSkipSigning ?? false,
        method: "POST",
        signerStateActionContext: {
          type: "tx-data",
          inputText: postInputText,
          signer: signerState.signer ?? null,
          frameContext,
          address: connectedAddress,
          url: homeframeUrl,
          target: frameButton.target,
          frameButton,
          buttonIndex,
          state: currentFrame.state
        },
        sourceFrame: currentFrame
      });
    },
    [
      fetchFrame,
      dangerousSkipSigning,
      frameContext,
      connectedAddress,
      homeframeUrl,
      signerState
    ]
  );
  const onButtonPress = (0, import_react2.useCallback)(
    async function onButtonPress2(currentFrame, frameButton, index, fetchFrameOverride = fetchFrame) {
      const clientSideActions = ["mint", "link"];
      const buttonRequiresAuth = !clientSideActions.includes(
        frameButton.action
      );
      if (!signerState.hasSigner && !dangerousSkipSigning && buttonRequiresAuth) {
        await signerState.onSignerlessFramePress();
        return;
      }
      switch (frameButton.action) {
        case "link": {
          try {
            validateLinkButtonTarget(frameButton.target);
          } catch (error) {
            if (error instanceof Error) {
              onErrorRef.current?.(error);
            }
            return;
          }
          onLinkButtonClick(frameButton);
          break;
        }
        case "mint": {
          onMint({
            frameButton,
            target: frameButton.target,
            frame: currentFrame
          });
          break;
        }
        case "tx": {
          await onTransactionButton({
            frameButton,
            buttonIndex: index + 1,
            postInputText: currentFrame.inputText !== void 0 ? inputText : void 0,
            currentFrame
          });
          break;
        }
        case "post":
        case "post_redirect": {
          try {
            const target = frameButton.target || frameButton.post_url || currentFrame.postUrl || homeframeUrl;
            if (!target) {
              onErrorRef.current?.(new Error(`Missing target`));
              return;
            }
            try {
              validateLinkButtonTarget(target);
            } catch (error) {
              if (error instanceof Error) {
                onErrorRef.current?.(error);
              }
              return;
            }
            await onPostButton({
              currentFrame,
              frameButton,
              /** https://docs.farcaster.xyz/reference/frames/spec#handling-clicks
              
                      POST the packet to fc:frame:button:$idx:action:target if present
                      POST the packet to fc:frame:post_url if target was not present.
                      POST the packet to or the frame's embed URL if neither target nor action were present.
                      */
              target,
              buttonIndex: index + 1,
              postInputText: currentFrame.inputText !== void 0 ? inputText : void 0,
              state: currentFrame.state,
              fetchFrameOverride
            });
            setInputText("");
          } catch (err) {
            if (err instanceof Error) {
              onErrorRef.current?.(err);
            }
            console.error(err);
          }
          break;
        }
        default:
          throw new Error("Unrecognized frame button action");
      }
    },
    [
      dangerousSkipSigning,
      fetchFrame,
      homeframeUrl,
      inputText,
      onLinkButtonClick,
      onMint,
      onPostButton,
      onTransactionButton,
      signerState
    ]
  );
  const clearFrameStack = (0, import_react2.useCallback)(() => {
    dispatch({ action: "CLEAR" });
  }, [dispatch]);
  const onCastActionButtonPress = (0, import_react2.useCallback)(
    async function onActionButtonPress(arg) {
      if (!signerState.hasSigner && !dangerousSkipSigning) {
        await signerState.onSignerlessFramePress();
        return;
      }
      return fetchFrame(
        {
          method: "CAST_ACTION",
          action: arg.castAction,
          isDangerousSkipSigning: dangerousSkipSigning ?? false,
          signerStateActionContext: {
            signer: signerState.signer ?? null,
            frameContext,
            url: arg.castAction.url,
            target: arg.castAction.url,
            buttonIndex: 1
          }
        },
        arg.clearStack
      );
    },
    [dangerousSkipSigning, fetchFrame, frameContext, signerState]
  );
  const onComposerActionButtonPress = (0, import_react2.useCallback)(
    async function onActionButtonPress(arg) {
      if (!signerState.hasSigner && !dangerousSkipSigning) {
        await signerState.onSignerlessFramePress();
        return;
      }
      return fetchFrame(
        {
          method: "COMPOSER_ACTION",
          action: arg.castAction,
          isDangerousSkipSigning: dangerousSkipSigning ?? false,
          composerActionState: arg.composerActionState,
          signerStateActionContext: {
            signer: signerState.signer ?? null,
            frameContext,
            url: arg.castAction.url,
            target: arg.castAction.url,
            buttonIndex: 1
          }
        },
        arg.clearStack
      );
    },
    [dangerousSkipSigning, fetchFrame, frameContext, signerState]
  );
  return (0, import_react2.useMemo)(() => {
    return {
      inputText,
      setInputText,
      clearFrameStack,
      dispatchFrameStack: dispatch,
      onButtonPress,
      fetchFrame,
      homeframeUrl,
      framesStack,
      currentFrameStackItem: framesStack[0],
      onCastActionButtonPress,
      onComposerActionButtonPress
    };
  }, [
    inputText,
    clearFrameStack,
    dispatch,
    onButtonPress,
    fetchFrame,
    homeframeUrl,
    framesStack,
    onCastActionButtonPress,
    onComposerActionButtonPress
  ]);
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  useFrame
});
